#!/usr/bin/perl

use Cwd 'abs_path';
use File::Copy;
use POSIX;

# Edit these variables (vid,pid,name) according to your's device characteristics.
# for example, this record already present in data base (and may be more then once)
$vid='0x2A0C';
$pid='0x0001';
$name='MultiSoft SCR2';


# do not edit following text

unless(geteuid()==0){
	die "\nroot permissions required";
}

$dir=`pkg-config libpcsclite --variable=usbdropdir`;
chomp $dir;
unless(-d $dir){
	die "\npcsc-lite not found";
}

$host=`uname | sed -e s,/,_,`;
chomp $host;
unless($host){
	die "\nunknown host";
}

$ENV{'PCSCLITE_HP_DROPDIR'}=$dir; # export 'directory name' to shell, where 'find' will run

$result = `find \$PCSCLITE_HP_DROPDIR -name libccid.so -print`;
@libccid_dirs = split(/\n/,$result); # may be more then one
$sz = @libccid_dirs; 
if($sz==1) {
	$libccid_dir=$libccid_dirs[0]; # but must be only one
	print 'libccid.so found: ', $libccid_dir, "\n";
} elsif($sz>1){
	die "\nmore then one libccid.so found";
} else {
	die "\nlibccid.so not found";
}

if( $libccid_dir =~ m{(.+)(/Contents)(/$host/libccid\.so)} ) {
	$info_plist_dir = $1.$2;
 	print 'Info.plist dir: ', $info_plist_dir, "\n";
} else {
	die "\nInfo.plist dir not found";
}

$info_plist = $info_plist_dir.'/Info.plist';
$real_info_plist=$info_plist;
# but in Debian, plist-file is a symlink. Check:
if(-l $info_plist){
	$real_info_plist=abs_path(readlink($info_plist));
	print 'file ', $info_plist, ' is a symlink to ', $real_info_plist, "\n";
}

open(F_inp,$real_info_plist) or die "\nunable open $real_info_plist";
@content = <F_inp>; # eat all file content
close(F_inp);

@tmp=@content;

@vids=();
@pids=();
@names=();

$vid_flag=0;
$pid_flag=0;
$name_flag=0;

$flag=1;
M1:while(@tmp){
	$line = shift @tmp;
	if($flag==2){goto M2;}
	if($flag==3){goto M3;}
	if( $vid_flag==0 ){if( $line !~ m{\<key\>ifdVendorID\<\/key\>} ){next M1;} else {$vid_flag=1;}}
	if( $line =~ m{\<array\>} ){next M1;}
	if( $line =~ m{\<\/array\>} ){$flag=2;next M1;}
	if( $line =~ m{\<string\>(.+)\<\/string\>} ){
		push @vids, $1;
	}
M2: 
	if( $pid_flag==0 ){if( $line !~ m{\<key\>ifdProductID\<\/key\>} ){next M1;} else {$pid_flag=1;}}
	if( $line =~ m{\<array\>} ){next M1;}
	if( $line =~ m{\<\/array\>} ){$flag=3;next M1;}
	if( $line =~ m{\<string\>(.+)\<\/string\>} ){
		push @pids, $1;
	}
M3:
	if( $name_flag==0 ){if( $line !~ m{\<key\>ifdFriendlyName\<\/key\>} ){next M1;} else {$name_flag=1;}}
	if( $line =~ m{\<array\>} ){next M1;}
	if( $line =~ m{\<\/array\>} ){last;}
	if( $line =~ m{\<string\>(.+)\<\/string\>} ){
		push @names, $1;
	}
}
$sz_vids=@vids;
$sz_pids=@pids;
$sz_names=@names;
if($sz_vids==$sz_pids and $sz_vids==$sz_names){
	print "vid:pid:name arrays have the same size, ok\n";
} else {
	die "\nError: vid:pid:name array's sizes mismatch";
}

$v2=lc($vid);
$p2=lc($pid);

# if more then one pair exists, - error
$pair_count=0;
$i=0;
print "find vid:pid duplicates...\n";
print "idx\t\tVID\t\tPID\t\tNAME\n";
for(;$i<$sz_vids;$i++){
	my $v1=lc($vids[$i]);
	my $p1=lc($pids[$i]);
	if(($v1 eq $v2) and ($p1 eq $p2)){
		++$pair_count;
		print $i,"\t\t",$vids[$i],"\t\t",$pids[$i],"\t\t",$names[$i],"\n";
	}
}
if($pair_count>1) {
# HERE!
#	die "\nmore then one pair with the same vid:pid found in data base";
}

$status_flag=3;
$i=0;
for(;$i<$sz_vids;$i++){
	my $v1=lc($vids[$i]);
	my $p1=lc($pids[$i]);
	if(($v1 eq $v2) and ($p1 eq $p2)){
		if($names[$i] eq $name){$status_flag=1;last;} else {$status_flag=2;last;}
	}
}
if($status_flag==3){
	print 'New record: VID:', $vid, ' PID:',$pid, ' NAME:',$name,"\n";
} elsif($status_flag==1){
	print 'record already exists: VID:', $vids[$i], ' PID:',$pids[$i], ' NAME:',$names[$i],"\n";
	exit 0;
} else {
	$k=$i;
	print 'record with different name: VID:', $vids[$i], ' PID:',$pids[$i], ' NAME:',$names[$i], ' (MUST:',$name,')',"\n";
}

# create reserved copy
($sec,$min,$hour,$day, $mon, $year) = (localtime)[0,1,2,3,4,5];

$bak_ext = sprintf "%04d.%02d.%02d_%02d:%02d:%02d",$year+1900,$mon+1,$day,$hour,$min,$sec;

$bak_file=$real_info_plist.'.'.$bak_ext;

print 'create bak-file: ', $bak_file, "\n";

copy($real_info_plist, $bak_file) or die "\ncreate bak-file: access denied";

# update orig file
$fout=$real_info_plist;
open(F_out,">$fout") or die "\nunable open $fout";
$sz_content=@content;
$update_flag=0;
M4:for($i=0;$i<$sz_content;$i++){
	if(!$update_flag){
		if( $content[$i] !~ m{\<key\>ifdVendorID\<\/key\>} ){
			print F_out $content[$i];
			next M4;
		}
	} else {
		if( $content[$i] !~ m{\<key\>Copyright\<\/key\>} ){
			next M4;
		}
		for($j=$i;$j<$sz_content;$j++){
			print F_out $content[$j];
		}
		last;
	}
	print F_out "\t",'<key>ifdVendorID</key>',"\n";
	print F_out "\t",'<array>',"\n";
	for($j=0;$j<$sz_vids;$j++){
			print F_out "\t\t",'<string>',$vids[$j],'</string>',"\n";
	}
	if($status_flag==3){
			print F_out "\t\t",'<string>',$vid,'</string>',"\n";
	}
	print F_out "\t",'</array>',"\n";
	print F_out "\n";
	
	print F_out "\t",'<key>ifdProductID</key>',"\n";
	print F_out "\t",'<array>',"\n";
	for($j=0;$j<$sz_pids;$j++){
		print F_out "\t\t",'<string>',$pids[$j],'</string>',"\n";
	}	
	if($status_flag==3){
			print F_out "\t\t",'<string>',$pid,'</string>',"\n";
	}
	print F_out "\t",'</array>',"\n";
	print F_out "\n";

	print F_out "\t",'<key>ifdFriendlyName</key>',"\n";
	print F_out "\t",'<array>',"\n";
	for($j=0;$j<$sz_names;$j++){
		if($status_flag==2){
			if($j==$k){
				print F_out "\t\t",'<string>',$name,'</string>',"\n";
			} else {
				print F_out "\t\t",'<string>',$names[$j],'</string>',"\n";
			}
		} else {
			print F_out "\t\t",'<string>',$names[$j],'</string>',"\n";
		}
	}
	if($status_flag==3){
			print F_out "\t\t",'<string>',$name,'</string>',"\n";
	}
	print F_out "\t",'</array>',"\n";
	print F_out "\n";
	$update_flag=1;
}

print 'Reader data base is updated.',"\n";
print 'If new reader is not resolved, restart pcscd', "\n";

close(F_out);

